@@ -23,7 +23,7 @@ class Component():
         """
         return {}
 
-    def save_flagged(self, dir, label, data):
+    def save_flagged(self, dir, label, data, encryption_key):
         """
         Saves flagged data from component
         """
@@ -35,8 +35,8 @@ class Component():
         """
         return data
 
-    def save_flagged_file(self, dir, label, data):
-        file = processing_utils.decode_base64_to_file(data)
+    def save_flagged_file(self, dir, label, data, encryption_key):
+        file = processing_utils.decode_base64_to_file(data, encryption_key)
         old_file_name = file.name
         output_dir = os.path.join(dir, label)
         if os.path.exists(output_dir):
