@@ -118,7 +118,7 @@ class Label(OutputComponent):
             "label": {},
         }
 
-    def save_flagged(self, dir, label, data):
+    def save_flagged(self, dir, label, data, encryption_key):
         """
         Returns: (Union[str, Dict[str, number]]): Either a string representing the main category label, or a dictionary with category keys mapping to confidence levels.
         """
@@ -195,11 +195,11 @@ class Image(OutputComponent):
             raise ValueError("Unknown type: " + dtype + ". Please choose from: 'numpy', 'pil', 'file', 'plot'.")
         return out_y, coordinates
 
-    def save_flagged(self, dir, label, data):
+    def save_flagged(self, dir, label, data, encryption_key):
         """
         Returns: (str) path to image file
         """
-        return self.save_flagged_file(dir, label, data[0])
+        return self.save_flagged_file(dir, label, data[0], encryption_key)
 
 
 class Video(OutputComponent):
@@ -224,11 +224,11 @@ class Video(OutputComponent):
     def postprocess(self, y):
         return processing_utils.encode_file_to_base64(y, type="video")
 
-    def save_flagged(self, dir, label, data):
+    def save_flagged(self, dir, label, data, encryption_key):
         """
         Returns: (str) path to image file
         """
-        return self.save_flagged_file(dir, label, data)
+        return self.save_flagged_file(dir, label, data, encryption_key)
 
 
 class KeyValues(OutputComponent):
@@ -259,7 +259,7 @@ class KeyValues(OutputComponent):
             "key_values": {},
         }
     
-    def save_flagged(self, dir, label, data):
+    def save_flagged(self, dir, label, data, encryption_key):
         return json.dumps(data)
 
     def restore_flagged(self, data):
@@ -297,7 +297,7 @@ class HighlightedText(OutputComponent):
     def postprocess(self, y):
         return y
 
-    def save_flagged(self, dir, label, data):
+    def save_flagged(self, dir, label, data, encryption_key):
         return json.dumps(data)
 
     def restore_flagged(self, data):
@@ -340,11 +340,11 @@ class Audio(OutputComponent):
         else:
             raise ValueError("Unknown type: " + self.type + ". Please choose from: 'numpy', 'file'.")
 
-    def save_flagged(self, dir, label, data):
+    def save_flagged(self, dir, label, data, encryption_key):
         """
         Returns: (str) path to audio file
         """
-        return self.save_flagged_file(dir, label, data)
+        return self.save_flagged_file(dir, label, data, encryption_key)
 
 
 class JSON(OutputComponent):
@@ -373,7 +373,7 @@ class JSON(OutputComponent):
             "json": {},
         }
 
-    def save_flagged(self, dir, label, data):
+    def save_flagged(self, dir, label, data, encryption_key):
         return json.dumps(data)
 
     def restore_flagged(self, data):
@@ -428,11 +428,11 @@ class File(OutputComponent):
             "data": processing_utils.encode_file_to_base64(y, header=False)
         }
 
-    def save_flagged(self, dir, label, data):
+    def save_flagged(self, dir, label, data, encryption_key):
         """
         Returns: (str) path to image file
         """
-        return self.save_flagged_file(dir, label, data["data"])
+        return self.save_flagged_file(dir, label, data["data"], encryption_key)
 
 
 class Dataframe(OutputComponent):
@@ -489,7 +489,7 @@ class Dataframe(OutputComponent):
         else:
             raise ValueError("Unknown type: " + self.type + ". Please choose from: 'pandas', 'numpy', 'array'.")
 
-    def save_flagged(self, dir, label, data):
+    def save_flagged(self, dir, label, data, encryption_key):
         """
         Returns: (List[List[Union[str, float]]]) 2D array
         """
