@@ -1,7 +1,7 @@
 function gradio(config, fn, target, example_file_path) {
   target = $(target);
   target.html(`
-    <div class="share invisible">
+    <div class="share hidden">
       Live at <a class="share-link" target="_blank"></a>.
       <button class="share-copy">Copy Link</button>
     </div>
@@ -12,32 +12,35 @@ function gradio(config, fn, target, example_file_path) {
         <div class="input_interfaces">
         </div>          
         <div class="panel_buttons">
-          <input class="clear panel_button" type="reset" value="CLEAR">
-          <input class="submit panel_button" type="submit" value="SUBMIT"/>
+          <button class="clear panel_button">CLEAR</button>
+          <button class="submit panel_button">SUBMIT</button>
         </div>
       </div>
       <div class="panel output_panel">
-        <div class="loading invisible">
+        <div class="loading hidden">
           <img class="loading_in_progress" src="/static/img/logo_loading.gif">
           <img class="loading_failed" src="/static/img/logo_error.png">
         </div>
         <div class="output_interfaces">
         </div>
         <div class="panel_buttons">
-          <input class="interpret panel_button" type="button" value="INTERPRET"/>
-          <input class="screenshot panel_button left_panel_button" type="button" value="SCREENSHOT"/>
-          <input class="record panel_button right_panel_button" type="button" value="GIF"/>
-          <div class="screenshot_logo invisible">
+          <button class="interpret inactive panel_button">INTERPRET</button>
+          <button class="screenshot panel_button left_panel_button">SCREENSHOT</button>
+          <button class="record panel_button right_panel_button">GIF</button>
+          <div class="screenshot_logo hidden">
             <img src="/static/img/logo_inline.png">
             <button class='record_stop'>
-              <div class='record_square' style=''></div>
+              <div class='record_square'></div>
             </button>
           </div>
-          <input class="flag panel_button" type="button" value="FLAG"/>
+          <div class="flag panel_button inactive">
+            FLAG
+            <div class="dropcontent"></div>
+          </div>
         </div>
       </div>
     </div>
-    <div class="interpretation_explained invisible">
+    <div class="interpretation_explained hidden">
       <h4>Interpretation Legend <span class='close_explain'>&#10006;</span></h4>
       <div class='interpretation_legend'>
         <div>&larr; Decreased output score / confidence</div>
@@ -46,19 +49,31 @@ function gradio(config, fn, target, example_file_path) {
       <p>When you click Interpret, you can see how different parts of the input contributed to the final output. The legend above will highlight each of the input components as follows:</p>
       <ul></ul>
     </div>
-    <div class="examples invisible">
+    <div class="examples hidden">
       <h4>Examples</small></h4>
-      <button class="run_examples examples-content">Run All</button>
-      <button class="load_prev examples-content">Load Previous <em>(CTRL + &larr;)</em></button>
-      <button class="load_next examples-content">Load Next <em>(CTRL + &rarr;)</em></button>
-      <button class="order_similar examples-content embedding">Order by Similarity</button>
-      <button class="view_embeddings examples-content embedding">View Embeddings</button>
-      <button class="update_embeddings embeddings-content invisible">Update Embeddings</button>
-      <button class="view_examples embeddings-content invisible">View Examples</button>
-      <div class="pages invisible">Page:</div>
+      <div class="examples_control">
+        <div class="examples_control_left">
+          <button class="run_examples examples-content">Run All</button>
+          <button class="load_prev examples-content">Load Previous <small>CTRL + <span class="backward">&#10140;</span></small></button>
+          <button class="load_next examples-content">Load Next <small>CTRL + &#10140;</small></button>
+          <button class="order_similar examples-content embedding">Order by Similarity</button>
+          <button class="view_embeddings examples-content embedding">View Embeddings</button>
+          <button class="update_embeddings embeddings-content hidden">Update Embeddings</button>
+          <button class="view_examples embeddings-content hidden">View Examples</button>
+        </div>
+        <div class="examples_control_right">
+          <button class="table_examples">
+            <svg width="40" height="24"><rect x="0" y="0" width="40" height="6"></rect><rect x="0" y="9" width="40" height="6"></rect><rect x="0" y="18" width="40" height="6"></rect></svg>
+          </button>
+          <button class="gallery_examples current">
+            <svg width="40" height="24"><rect x="0" y="0" width="18" height="40"></rect><rect x="22" y="0" width="18" height="40"></rect></svg>
+          </button>
+        </div>
+      </div>
+      <div class="pages hidden">Page:</div>
       <table class="examples-content">
       </table>
-      <div class="plot embeddings-content invisible"><canvas id="canvas" width="400px" height="300px"></canvas></div>
+      <div class="plot embeddings-content hidden"><canvas id="canvas" width="400px" height="300px"></canvas></div>
     </div>
     <p class="article"></p>    
     `);
@@ -118,7 +133,7 @@ function gradio(config, fn, target, example_file_path) {
   }
   if (config["share_url"]) {
     let share_url = config["share_url"];
-    target.find(".share").removeClass("invisible");
+    target.find(".share").removeClass("hidden");
     target.find(".share-link").text(share_url).attr("href", share_url);
     target.find(".share-copy").click(function() {
       copyToClipboard(share_url);
@@ -191,55 +206,54 @@ function gradio(config, fn, target, example_file_path) {
     for (let output_interface of output_interfaces) {
       output_interface.clear();
     }
-    target.find(".flag").removeClass("flagged");
-    target.find(".flag").val("FLAG");
-    target.find(".flag_message").empty();
-    target.find(".loading").addClass("invisible");
+    target.find(".loading").addClass("hidden");
     target.find(".loading_time").text("");
     target.find(".output_interfaces").css("opacity", 1);
+    target.find(".flag").addClass("inactive");
+    target.find(".interpret").addClass("inactive");
     io_master.last_input = null;
     io_master.last_output = null;
   }  
   target.find(".clear").click(clear_all);
 
   if (!config["allow_embedding"]) {
-    target.find(".embedding").css("visibility", "hidden");
+    target.find(".embedding").hide();
   }
-  if (!config["allow_screenshot"] && config["allow_flagging"] !== true && !config["allow_interpretation"]) {
-    target.find(".screenshot, .record, .flag, .interpret").css("visibility", "hidden");
+  if (!config["allow_screenshot"]) {
+    target.find(".screenshot, .record").hide();
+  }
+  if (config["allow_flagging"] !== true) {
+    target.find(".flag").hide();
+  }
+  if (!config["allow_interpretation"]) {
+    target.find(".interpret").hide();
   } else {
-    if (!config["allow_screenshot"]) {
-      target.find(".screenshot, .record").hide();
-    }
-    if (config["allow_flagging"] !== true) {
-      target.find(".flag").hide();
-    }
-    if (!config["allow_interpretation"]) {
-      target.find(".interpret").hide();
-    } else {
-      let interpret_html = ""; 
-      for (let [i, interface] of io_master.input_interfaces.entries()) {
-        let label = config.input_interfaces[i][1]["label"];;
-        interpret_html += "<li><strong>" + label + "</strong> - " + interface.interpretation_logic + "</li>";
-      }
-      target.find(".interpretation_explained ul").html(interpret_html);
-      target.find(".interpretation_explained .close_explain").click(function() {
-        target.find(".interpretation_explained").remove();
-      });
-      if (config["examples"]) {
-        target.find(".examples").removeClass("invisible");
-        let html = "<thead>"
-        for (let i = 0; i < config["input_interfaces"].length; i++) {
-          label = config["input_interfaces"][i][1]["label"];
-          html += "<th>" + label + "</th>";
-        }
-      }
+    let interpret_html = ""; 
+    for (let [i, interface] of io_master.input_interfaces.entries()) {
+      let label = config.input_interfaces[i][1]["label"];
+      interpret_html += "<li><strong>" + label + "</strong> - " + interface.interpretation_logic + "</li>";
     }
+    target.find(".interpretation_explained ul").html(interpret_html);
+    target.find(".interpretation_explained .close_explain").click(function() {
+      target.find(".interpretation_explained").remove();
+    });
   }
   function load_example(example_id) {
     clear_all();
+    if (!(example_id in config["examples"])) {
+      return;
+    }
     for (let [i, value] of config["examples"][example_id].entries()) {
-      input_interfaces[i].load_example(value);
+      if (i < input_interfaces.length) {
+        input_interfaces[i].load_example(value);
+      } else if (i - input_interfaces.length < output_interfaces.length) {
+        let output_interface = output_interfaces[i - input_interfaces.length];
+        if ("load_example" in output_interface) {
+          output_interface.load_example(value);
+        } else {
+          output_interface.output(value)
+        }
+      }
     };
     if (io_master.loaded_examples && example_id in io_master.loaded_examples) {
       io_master.output({"data": io_master.loaded_examples[example_id]});
@@ -253,6 +267,17 @@ function gradio(config, fn, target, example_file_path) {
     $(".examples_body > tr").removeClass("current_example");
     $(".examples_body > tr[row='" + example_id + "'").addClass("current_example");
     io_master.current_example = example_id;
+    window.location.hash = example_id + 1;
+  }
+  function hash_handler() {
+    let hash = window.location.hash;
+    if (hash == "") {
+      return;
+    }
+    hash = hash.substring(1)
+    if (!isNaN(parseInt(hash))) {
+      load_example(parseInt(hash) - 1);
+    }
   }
   function next_example() {
     current_example = io_master.current_example;
@@ -284,8 +309,15 @@ function gradio(config, fn, target, example_file_path) {
       html += "<tr row=" + example_id + ">";
       for (let [j, col] of example.entries()) {
         let new_col = JSON.parse(JSON.stringify(col))
-        if (input_interfaces[j].load_example_preview) {
-          new_col = input_interfaces[j].load_example_preview(new_col);
+        if (j < input_interfaces.length) {
+          if (input_interfaces[j].load_example_preview) {
+            new_col = input_interfaces[j].load_example_preview(new_col);
+          }
+        } else {
+          let k = j - input_interfaces.length;
+          if (k < output_interfaces.length && output_interfaces[k].load_example_preview) {
+            new_col = output_interfaces[k].load_example_preview(new_col);
+          }
         }
         html += "<td>" + new_col + "</td>";
       }
@@ -305,11 +337,15 @@ function gradio(config, fn, target, example_file_path) {
     target.find(".examples > table > tbody").html(html);
   }
   if (config["examples"]) {
-    target.find(".examples").removeClass("invisible");
+    target.find(".examples").removeClass("hidden");
     let html = "<thead>"
-    for (let i = 0; i < config["input_interfaces"].length; i++) {
-      label = config["input_interfaces"][i][1]["label"];
-      html += "<th>" + label + "</th>";
+    for (let input_interface of config["input_interfaces"]) {
+      html += "<th>" + input_interface[1]["label"] + "</th>";
+    }
+    if (config["examples"].length > 0 && config["examples"][0].length > config["input_interfaces"].length) {
+      for (let output_interface of config["output_interfaces"]) {
+        html += "<th>" + output_interface[1]["label"] + "</th>";
+      }
     }
     html += "</thead>";
     html += "<tbody class='examples_body'></tbody>";
@@ -318,7 +354,7 @@ function gradio(config, fn, target, example_file_path) {
     io_master.order_mapping = [...Array(config.examples.length).keys()];
     let page_count = Math.ceil(config.examples.length / config.examples_per_page)
     if (page_count > 1) {
-      target.find(".pages").removeClass("invisible");
+      target.find(".pages").removeClass("hidden");
       let html = "";
       for (let i = 0; i < page_count; i++) {
         html += `<button class='page' page='${i}'>${i+1}</button>`
@@ -326,6 +362,8 @@ function gradio(config, fn, target, example_file_path) {
       target.find(".pages").append(html);
     }
     load_page();
+    window.onhashchange = hash_handler;
+    hash_handler();  
     target.on("click", ".examples_body > tr", function() {
       let example_id = parseInt($(this).attr("row"));
       load_example(example_id);
@@ -335,6 +373,23 @@ function gradio(config, fn, target, example_file_path) {
       io_master.current_page = page_num;
       load_page();
     })
+    set_table_mode = function() {
+      target.find(".examples-content").removeClass("gallery");
+      target.find(".examples_control_right button").removeClass("current");
+      target.find(".table_examples").addClass("current");
+    }
+    set_gallery_mode = function() {
+      target.find(".examples-content").addClass("gallery");
+      target.find(".examples_control_right button").removeClass("current");
+      target.find(".gallery_examples").addClass("current");
+    }
+    target.on("click", ".table_examples",  set_table_mode);
+    target.on("click", ".gallery_examples",  set_gallery_mode);
+    if (config["examples"].length > 0 && config["examples"][0].length > 1) {
+      set_table_mode();
+    } else {
+      set_gallery_mode();
+    }
     target.find(".load_prev").click(prev_example);
     target.find(".load_next").click(next_example);
     target.find(".order_similar").click(function() {
@@ -345,8 +400,8 @@ function gradio(config, fn, target, example_file_path) {
       })
     });
     target.find(".view_examples").click(function() {
-      target.find(".examples-content").removeClass("invisible");        
-      target.find(".embeddings-content").addClass("invisible");  
+      target.find(".examples-content").removeClass("hidden");        
+      target.find(".embeddings-content").addClass("hidden");  
     });
     target.find(".update_embeddings").click(function() {
       io_master.update_embeddings(function(output) {
@@ -393,8 +448,8 @@ function gradio(config, fn, target, example_file_path) {
           }
         };
     
-        target.find(".examples-content").addClass("invisible");        
-        target.find(".embeddings-content").removeClass("invisible");  
+        target.find(".examples-content").addClass("hidden");        
+        target.find(".embeddings-content").removeClass("hidden");  
         })
     });
     $("body").keydown(function(e) {
@@ -417,7 +472,7 @@ function gradio(config, fn, target, example_file_path) {
   
   target.find(".screenshot").click(function() {
     $(".screenshot, .record").hide();
-    $(".screenshot_logo").removeClass("invisible");
+    $(".screenshot_logo").removeClass("hidden");
     $(".record_stop").hide();
     html2canvas(target[0], {
       scrollX: 0,
@@ -425,15 +480,15 @@ function gradio(config, fn, target, example_file_path) {
     }).then(function(canvas) {
       saveAs(canvas.toDataURL(), 'screenshot.png');
       $(".screenshot, .record").show();
-      $(".screenshot_logo").addClass("invisible");
+      $(".screenshot_logo").addClass("hidden");
     });
   });
   target.find(".record").click(function() {
     $(".screenshot, .record").hide();
-    $(".screenshot_logo").removeClass("invisible");
+    $(".screenshot_logo").removeClass("hidden");
     $(".record_stop").show();
-    target.append("<canvas class='recording_draw invisible' width=640 height=480></canvas>");
-    target.append("<video class='recording invisible' autoplay playsinline></video>");
+    target.append("<canvas class='recording_draw hidden' width=640 height=480></canvas>");
+    target.append("<video class='recording hidden' autoplay playsinline></video>");
     navigator.mediaDevices.getDisplayMedia(
       { video: { width: 9999, height: 9999 } }
     ).then(stream => {
@@ -488,7 +543,7 @@ function gradio(config, fn, target, example_file_path) {
     processFrame(start);
 
     $(".screenshot, .record").show();
-    $(".screenshot_logo").addClass("invisible");
+    $(".screenshot_logo").addClass("hidden");
     target.find("canvas.recording_draw").remove();
     target.find("video.recording").remove();
   })
@@ -498,23 +553,48 @@ function gradio(config, fn, target, example_file_path) {
     target.find(".submit").show();
     target.find(".submit").click(function() {
       io_master.gather();
-      target.find(".flag").removeClass("flagged");
-      target.find(".flag").val("FLAG");
     })
   }
   if (!config.show_input) {
     target.find(".input_panel").hide();
   }
+  function flash_flag() {
+    target.find(".flag").addClass("flagged");
+    target.find(".dropcontent").addClass("hidden");
+    window.setTimeout(() => {
+      target.find(".flag").removeClass("flagged");
+      target.find(".dropcontent").removeClass("hidden");
+    }, 500);
 
-  target.find(".flag").click(function() {
-    if (io_master.last_output) {
-      target.find(".flag").addClass("flagged");
-      target.find(".flag").val("FLAGGED");
-      io_master.flag();
+  }
+  if (config.allow_flagging) {
+    if (config.flagging_options) {
+      target.find(".flag").addClass("dropdown");
+      for (let option of config.flagging_options) {
+        target.find(".dropcontent").append(`<div>${option}</div>`)
+      }
+      target.find(".flag .dropcontent div").click(function() {
+        if (io_master.last_output) {
+          target.find(".flag .dropcontent");
+          flash_flag();
+          io_master.flag($(this).text());
+        }
+      });
+      
+    } else {
+      target.find(".flag").click(function() {
+        if (io_master.last_output) {
+          flash_flag();
+          io_master.flag();
+        }
+      });
     }
-  })
+  }
+  if (config.hide_run_all) {
+    $(".run_examples").hide();
+  }
   target.find(".interpret").click(function() {
-    target.find(".interpretation_explained").removeClass("invisible");
+    target.find(".interpretation_explained").removeClass("hidden");
     if (io_master.last_output) {
       io_master.interpret();
     }
